// SETA43
// http://www.seta43.netau.net/electro.html
// Basado en  http://playground.arduino.cc/Main/DS1302
// 5-03-2013

#include <avr/io.h>
#include <avr/interrupt.h>

#include <avr/sleep.h>

#include <LiquidCrystal.h>

#define debug 0


#define DS1302_SCLK 15    // Arduino pin for the Serial Clock
#define DS1302_IO   16    // Arduino pin for the Data I/O
#define DS1302_CE   17    // Arduino pin for the Chip Enable


#define DS1302_SECONDS           0x80
#define DS1302_MINUTES           0x82
#define DS1302_HOURS             0x84
#define DS1302_DATE              0x86
#define DS1302_MONTH             0x88
#define DS1302_DAY               0x8A
#define DS1302_YEAR              0x8C
#define DS1302_ENABLE            0x8E
#define DS1302_TRICKLE           0x90
#define DS1302_CLOCK_BURST       0xBE
#define DS1302_CLOCK_BURST_WRITE 0xBE
#define DS1302_CLOCK_BURST_READ  0xBF
#define DS1302_RAMSTART          0xC0
#define DS1302_RAMEND            0xFC
#define DS1302_RAM_BURST         0xFE
#define DS1302_RAM_BURST_WRITE   0xFE
#define DS1302_RAM_BURST_READ    0xFF

#define DS1302_D0 0
#define DS1302_D1 1
#define DS1302_D2 2
#define DS1302_D3 3
#define DS1302_D4 4
#define DS1302_D5 5
#define DS1302_D6 6
#define DS1302_D7 7

#define DS1302_READBIT DS1302_D0 // READBIT=1: read instruction
#define DS1302_RC DS1302_D6
#define DS1302_CH DS1302_D7   // 1 = Clock Halt, 0 = start
#define DS1302_AM_PM DS1302_D5 // 0 = AM, 1 = PM
#define DS1302_12_24 DS1302 D7 // 0 = 24 hour, 1 = 12 hour

#define DS1302_WP DS1302_D7   // 1 = Write Protect, 0 = enabled

#define DS1302_ROUT0 DS1302_D0
#define DS1302_ROUT1 DS1302_D1
#define DS1302_DS0   DS1302_D2
#define DS1302_DS1   DS1302_D2
#define DS1302_TCS0  DS1302_D4
#define DS1302_TCS1  DS1302_D5
#define DS1302_TCS2  DS1302_D6
#define DS1302_TCS3  DS1302_D7




typedef struct ds1302_struct
{
  uint8_t Seconds:4;      // low decimal digit 0-9
  uint8_t Seconds10:3;    // high decimal digit 0-5
  uint8_t CH:1;       // CH = Clock Halt
  
  uint8_t Minutes:4;
  uint8_t Minutes10:3;
  uint8_t reserved1:1;
  
  union
  {
    struct
    {
      uint8_t Hour:4;
      uint8_t Hour10:2;
      uint8_t reserved2:1;
      uint8_t hour_12_24:1; // 0 for 24 hour format
    } h24;
    struct
    {
      uint8_t Hour:4;
      uint8_t Hour10:1;
      uint8_t AM_PM:1;      // 0 for AM, 1 for PM
      uint8_t reserved2:1;
      uint8_t hour_12_24:1; // 1 for 12 hour format
    } h12;
  };
  uint8_t Date:4;
  uint8_t Date10:2;
  uint8_t reserved3:2;
  
  uint8_t Month:4;
  uint8_t Month10:1;
  uint8_t reserved4:3;
  
  uint8_t Day:3;
  uint8_t reserved5:5;
  
  uint8_t Year:4;
  uint8_t Year10:4;
  
  uint8_t reserved6:7;
  uint8_t WP:1;             // WP = Write Protect
};


LiquidCrystal lcd(2,3,4,5,6,7);

byte LedPin=13;
byte LedON=11;
byte e1Pin=9;
byte e2Pin=10;

byte horas,minutos,segundos,segundos2,dia,mes;
int ano;
byte Ahoras,Aminutos,Asegundos;
byte bandera=1;
byte bandera2;
int dormir;
byte alarma;  //0-1 Alarma ON, 3-Alarma OFF

 char buffer[80];     // the code uses 70 characters.
 ds1302_struct rtc;
 
 char *menu[]={" ","SLEEP","ALARM","TIME","DATE"};
 byte mymenu;
 
#define VELOCIDAD 10

 void setup()
 {      
  Serial.begin(9600);
  lcd.begin(16, 2);
  
  pinMode(LedPin, OUTPUT);
  pinMode(LedON, OUTPUT);
  pinMode(e1Pin, INPUT);
  pinMode(e2Pin, INPUT);

  DS1302_write (DS1302_ENABLE, 0);
  DS1302_write (DS1302_TRICKLE, 0x00);

//Descomentar para ajustar la hora la primera vez
//#define SET_DATE_TIME_JUST_ONCE
#ifdef SET_DATE_TIME_JUST_ONCE  
  memset ((char *) &rtc, 0, sizeof(rtc));
  rtc.Seconds = 0;
  rtc.Seconds10 = 0;
  rtc.CH = 0;        // 1 for Clock Halt, 0 to run;
  rtc.Minutes = 4;
  rtc.Minutes10 = 0;
  rtc.h24.Hour = 3;
  rtc.h24.Hour10 = 1;
  rtc.h24.hour_12_24 = 0; // 0 for 24 hour format
  rtc.Date = 4;
  rtc.Date10 = 0;
  rtc.Month = 3;
  rtc.Month10 = 0;
  rtc.Day = 2;
  rtc.Year = 3;
  rtc.Year10 = 1;
  rtc.WP = 0;  
  DS1302_clock_burst_write( (uint8_t *) &rtc);
#endif

//Si se quiere que la frecuencia varie, descomenta la linea siguiente
//#define FREVAR

watchdogOn(); // Turn on the watch dog timer.

ADCSRA = ADCSRA & B01111111;
ACSR = B10000000;

//DIDR0 = DIDR0 | B00111111;


  mymenu=0;
  dormir=0;
  alarma=3;
  
  Ahoras=DS1302_read(0xc1);
  Aminutos=DS1302_read(0xc3);
  alarma=DS1302_read(0xc5);

  lcd.setCursor(0,0);
  lcd.print("RELOJ2");
  

}


void loop()
{
  
  if(mymenu==0)goToSleep();
  if(!digitalRead(e1Pin))
        {
         mymenu++;         
         if(mymenu>4)mymenu=0;
         lcd.clear();
         lcd.setCursor(0,0);
         lcd.print(menu[mymenu]);
         if(bandera2==1)mymenu=0;
         bandera2=0;
         while(!digitalRead(e1Pin));
         delay(VELOCIDAD);
        }
    if(!digitalRead(e2Pin))
        {
         if(dormir>0)
             {              
              dormir=0; 
             }
            else
             {
              dormir=59;
             }
         while(!digitalRead(e2Pin));
         delay(VELOCIDAD);
        }
    if(alarma==1) { dormir=10; alarma=0;}
    if(dormir<1)
               {
                digitalWrite(LedON,LOW);               
               }
             else
              {               
                if(!digitalRead(LedON))
                   {
                    digitalWrite(LedON,HIGH);
                   }
              }

#ifdef  FREVAR 
    if(mymenu!=0)
         {
          CLKPR = (1 << CLKPCE); // Velocidad 16mhz
          CLKPR = 0;       
         }
#endif
         
    switch(mymenu)
    {      
      case 0:     
         if(bandera)
          {
           #ifdef  FREVAR
           CLKPR = (1 << CLKPCE); // Velocidad 16mhz
           CLKPR = 0;  
          #endif
           bandera=0;                                       
           lcd.setCursor(10,0);
           if(segundos & 4)
                         lcd.print("SETA43");
                     else
                         lcd.print("DS1302"); 
           
           digitalWrite(LedPin, segundos & 1);    
           sprintf(buffer,"%02d:%02d:%02d",horas,minutos,segundos); 
           if(debug)Serial.println(buffer);
           lcd.setCursor(0,0);
           lcd.print(buffer); 
           
           if(alarma!=3)
             {
               sprintf(buffer,"A%02d:%02d",Ahoras,Aminutos); 
               if(debug)Serial.println(buffer);
               lcd.setCursor(10,1);
               lcd.print(buffer); 
             }
        
           sprintf(buffer, "%d-%d-%d  ",dia,mes,ano);
           if(debug) Serial.println(buffer);
           lcd.setCursor(0,1);
           lcd.print(buffer);
           #ifdef  FREVAR
            CLKPR = (1 << CLKPCE); // Velocidad 500khz
            CLKPR = 5; 
           #endif
           
          }
      break;
      case 1:
         lcd.setCursor(0,1);
         lcd.print(dormir);
         lcd.print(" Minutos ");
         if(!digitalRead(e2Pin))
             {
               if(dormir>11)
                   dormir-=10;
                  else
                   dormir--;
               if(dormir<0)dormir=60;
               bandera2=1;
               while(!digitalRead(e2Pin));
             }         
      break;      
      case 2:
         sprintf(buffer,"%02d:%02d",Ahoras,Aminutos);          
         lcd.setCursor(0, 1);
         lcd.print(buffer); 
          if(!digitalRead(e2Pin))
             {             
              menuAlarm();
              mymenu=0;
              lcd.clear();
               while(!digitalRead(e2Pin));
             }       
      break;
      case 3:
          sprintf(buffer,"%02d:%02d",horas,minutos);          
          lcd.setCursor(0, 1);
          lcd.print(buffer); 
          if(!digitalRead(e2Pin))
             {              
              menuTime();
              mymenu=0;
              lcd.clear();
              while(!digitalRead(e2Pin));
             }       
      break;
      case 4:
          sprintf(buffer, "%d-%d-%d  ",dia,mes,ano);          
          lcd.setCursor(0, 1);
          lcd.print(buffer); 
          if(!digitalRead(e2Pin))
             {              
              menuDate();
              mymenu=0;
              lcd.clear();
              while(!digitalRead(e2Pin));
             }       
      break;    
    }             

}

//------------------------------------------
void goToSleep()   
{
set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode.
sleep_enable(); // Enable sleep mode.
sleep_mode(); // Enter sleep mode.
sleep_disable(); // Disable sleep mode after waking.
                     
}


void watchdogOn() {
//Fuente http://www.fiz-ix.com/2012/11/low-power-arduino-using-the-watchdog-timer/
// Clear the reset flag, the WDRF bit (bit 3) of MCUSR.
MCUSR = MCUSR & B11110111;
WDTCSR = WDTCSR | B00011000; 
WDTCSR = B00000101; //0.5S
//WDTCSR = B00000110; //1S
//WDTCSR = B00100001; //8S

// Enable the watchdog timer interupt.
WDTCSR = WDTCSR | B01000000;
MCUSR = MCUSR & B11110111;

}

ISR(WDT_vect)
{
    #ifdef  FREVAR
      CLKPR = (1 << CLKPCE); 
      CLKPR = 0;      
    #endif
  
  DS1302_clock_burst_read( (uint8_t *) &rtc);
  
  segundos2=(rtc.Seconds10 * 10) + rtc.Seconds;
  if(segundos!=segundos2)
    {
     bandera=1;
     if(segundos==0)if(dormir>0)dormir--;
     segundos=segundos2;
     minutos=(rtc.Minutes10 * 10) + rtc.Minutes;
     horas=(rtc.h24.Hour10 * 10) + rtc.h24.Hour;
     dia=(rtc.Date10 * 10) + rtc.Date;
     mes=(rtc.Month10 * 10) + rtc.Month;
     ano=2000+(rtc.Year10 * 10) + rtc.Year;
     if(alarma==0 && segundos==0 && minutos==Aminutos && horas==Ahoras)alarma=1; 
    }
    
   #ifdef  FREVAR
     CLKPR = (1 << CLKPCE); // Velocidad 500khz
     CLKPR = 5;
   #endif
}


//-----------------------------------------
void menuDate()
{
 byte d1,d,m1,m,a1,a;
   cli();
   while(1)
    {
     lcd.setCursor(0,1);
     lcd.print("Day     ");
     lcd.print(dia);
     lcd.print("  "); 
     if(!digitalRead(e2Pin))
             {
              dia++;
              if(dia>31)dia=1; 
              while(!digitalRead(e2Pin));
             }
     if(!digitalRead(e1Pin))
            {
              while(!digitalRead(e1Pin));
              break;         
            }  
    }
   while(1)
    {
     lcd.setCursor(0,1);
     lcd.print("Month   ");
     lcd.print(mes);
     lcd.print("  "); 
     if(!digitalRead(e2Pin))
             {
              mes++;
              if(mes>12)mes=1; 
              while(!digitalRead(e2Pin));
             }
     if(!digitalRead(e1Pin))
            {
              while(!digitalRead(e1Pin));
              break;         
            }  
    }
   ano-=2000; 
   while(1)
    {
     lcd.setCursor(0,1);
     lcd.print("Year     ");
     lcd.print(ano+2000);
     lcd.print("  "); 
     if(!digitalRead(e2Pin))
             {
              ano++;
              if(ano>50)ano=00; 
              while(!digitalRead(e2Pin));
             }
     if(!digitalRead(e1Pin))
            {
              while(!digitalRead(e1Pin));
              break;         
            }  
    }
  d1=dia/10; d=dia-(d1*10);
  d1=d1<<4; d1=d1 | d;
  DS1302_write(0x86,d1);
  
  m1=mes/10; m=mes-(m1*10);
  m1=m1<<4; m1=m1 | m;
  DS1302_write(0x88,m1);
  
  a1=ano/10; a=ano-(a1*10);
  a1=a1<<4; a1=a1 | a;
  DS1302_write(0x8c,a1);
  sei();
}

void menuTime()
{
  byte h1,h,m1,m;
  cli();
  while(1)
    {
     lcd.setCursor(0,1);
     lcd.print("Hours   ");
     lcd.print(horas);
     lcd.print("  "); 
     if(!digitalRead(e2Pin))
             {
              horas++;
              if(horas>23)horas=0; 
              while(!digitalRead(e2Pin));
             }
     if(!digitalRead(e1Pin))
            {
              while(!digitalRead(e1Pin));
              break;         
            }  
    }
  while(1)
    {
      lcd.setCursor(0,1);
      lcd.print("Minutes ");
      lcd.print(minutos);
      lcd.print("  "); 
      if(!digitalRead(e2Pin))
             {
              minutos++;
              segundos=0;
              if(minutos>59)minutos=0;
              while(!digitalRead(e2Pin));
             }
      if(!digitalRead(e1Pin))
            {
              while(!digitalRead(e1Pin));
              break;         
            }  
    }    
  h1=horas/10; h=horas-(h1*10);
  h1=h1<<4; h1=h1 | h;
  DS1302_write(0x84,h1);
  
  m1=minutos/10; m=minutos-(m1*10);
  m1=m1<<4; m1=m1 | m;
  DS1302_write(0x82,m1);
  
  DS1302_write(0x80,0); //Clear segundos
  
  sei();  
}


void menuAlarm()
{
   while(1)
    {
     lcd.setCursor(0,1);
     lcd.print("Alarm   ");
     if(alarma==3)
         lcd.print("OFF");
        else
         lcd.print("ON ");
     if(!digitalRead(e2Pin))
             {
              if(alarma==3)
                  alarma=0;
                else
                  alarma=3;                              
              while(!digitalRead(e2Pin));
             }
     if(!digitalRead(e1Pin))
            {
              while(!digitalRead(e1Pin));
              break;         
            }  
    }
  
   while(1)
    {
     lcd.setCursor(0,1);
     lcd.print("Hours   ");
     lcd.print(Ahoras);
     lcd.print("  "); 
     if(!digitalRead(e2Pin))
             {
              Ahoras++;
              if(Ahoras>23)Ahoras=0; 
              while(!digitalRead(e2Pin));
             }
     if(!digitalRead(e1Pin))
            {
              while(!digitalRead(e1Pin));
              break;         
            }  
    }
  while(1)
    {
      lcd.setCursor(0,1);
      lcd.print("Minutes ");
      lcd.print(Aminutos);
      lcd.print("  "); 
      if(!digitalRead(e2Pin))
             {
              Aminutos++;
              Asegundos=0;
              if(Aminutos>59)Aminutos=0;
              while(!digitalRead(e2Pin));
             }
      if(!digitalRead(e1Pin))
            {
              while(!digitalRead(e1Pin));
              break;         
            }  
    } 
   DS1302_write(0xc0,Ahoras);   
   DS1302_write(0xc2,Aminutos);
   DS1302_write(0xc4,alarma);   
}  



//--------------------------------------

void DS1302_clock_burst_read( uint8_t *p)
{
  int i;

  _DS1302_start();
  _DS1302_togglewrite( DS1302_CLOCK_BURST_READ, true);  
  for (i=0; i<8; i++)    *p++ = _DS1302_toggleread();  
  _DS1302_stop();
}


void DS1302_clock_burst_write( uint8_t *p)
{
  int i;

  _DS1302_start();
  _DS1302_togglewrite( DS1302_CLOCK_BURST_WRITE, false);  
  for (i=0; i<8; i++)    _DS1302_togglewrite( *p++, false);  
  _DS1302_stop();
}

uint8_t DS1302_read(int address)
{
  uint8_t data;

  bitSet (address, DS1302_READBIT);  
  _DS1302_start();
  _DS1302_togglewrite (address, true);  
  data = _DS1302_toggleread ();
  _DS1302_stop();

  return (data);
}

void DS1302_write(int address, uint8_t data)
{
  bitClear (address, DS1302_READBIT);  

  _DS1302_start();
  _DS1302_togglewrite (address, false);
  _DS1302_togglewrite (data, false);
  _DS1302_stop();  
}

void _DS1302_start(void)
{
  digitalWrite (DS1302_CE, LOW); // default, not enabled
  pinMode (DS1302_CE, OUTPUT);  
  digitalWrite (DS1302_SCLK, LOW); // default, clock low
  pinMode (DS1302_SCLK, OUTPUT);
  pinMode (DS1302_IO, OUTPUT);
  digitalWrite (DS1302_CE, HIGH); // start the session
  delayMicroseconds(4);           // tCC = 4us
}

void _DS1302_stop(void)
{
  digitalWrite (DS1302_CE, LOW);
  delayMicroseconds(4);           // tCWH = 4us
}


uint8_t _DS1302_toggleread(void)
{
  uint8_t i, data;

  data = 0;
  for (i = 0; i <= 7; i++)
  {
    digitalWrite (DS1302_SCLK, HIGH);
    delayMicroseconds(1);
    digitalWrite (DS1302_SCLK, LOW);
    delayMicroseconds(1);        // tCL=1000ns, tCDD=800ns
    bitWrite (data, i, digitalRead(DS1302_IO));
  }
  return (data);
}


void _DS1302_togglewrite(uint8_t data, uint8_t release)
{
  int i;

  for (i = 0; i <= 7; i++)
  {
    digitalWrite (DS1302_IO, bitRead(data, i));  
    delayMicroseconds(1);     // tDC = 200ns
    digitalWrite (DS1302_SCLK, HIGH);    
    delayMicroseconds(1);     // tCH = 1000ns, tCDH = 800ns
    if (release && i == 7)
    {
      pinMode (DS1302_IO, INPUT);
      digitalWrite (DS1302_IO, LOW); // remove any pull-up  
    }
    else
    {
      digitalWrite (DS1302_SCLK, LOW);
      delayMicroseconds(1);       // tCL=1000ns, tCDD=800ns
    }
  }
}
